<?php
namespace Revenue;

/**
 * Handle Revenue Compatitbility
 *
 * @hooked on init
 */

/**
 * Handles compatibility with other Plugins
 *
 * @class    Revenue_Compatibility
 * @version  1.0.0
 */
class Revenue_Compatibility {
	use SingletonTrait;

	/**
	 * Min required plugin versions to check.
	 *
	 * @var array
	 */
	private $required = array();

	/**
	 * Modules to load.
	 *
	 * @var array
	 */
	private $modules = array();

	public function init() {
		// Define dependencies.
		$this->required = array(
			'blocks' => '7.2.0',
		);

		// Initialize.
		$this->load_modules();
	}
	/**
	 * Initialize.
	 *
	 * @since  1.0.0
	 *
	 * @return void
	 */
	protected function load_modules() {
		// Load modules.
		add_action( 'plugins_loaded', array( $this, 'module_includes' ), 100 );
	}


	/**
	 * Load compatibility classes.
	 *
	 * @return void
	 */
	public function module_includes() {

		$module_paths = array();

		// WooCommerce Cart/Checkout Blocks support.
		if ( class_exists( 'Automattic\WooCommerce\Blocks\Package' ) && version_compare( \Automattic\WooCommerce\Blocks\Package::get_version(), $this->required['blocks'] ) >= 0 ) {
			$module_paths['blocks'] = REVENUE_PATH . 'includes/compatibility/modules/class-revenue-blocks-compatibility.php';
		}

		/**
		 * 'revenue_compatibility_modules' filter.
		 *
		 * Use this to filter the required compatibility modules.
		 *
		 * @since  1.0.0
		 * @param  array $module_paths
		 */
		$this->modules = apply_filters( 'revenue_compatibility_modules', $module_paths );

		foreach ( $this->modules as $name => $path ) {
			if ( file_exists( $path ) ) {
				require_once $path;
			}
		}
	}
}
